@@ -31,7 +31,7 @@
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-shadow.c,v 1.3 2001/01/10 07:30:21 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-shadow.c,v 1.5 2001/01/31 01:16:44 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -71,7 +71,7 @@ gdm_verify_user (const gchar *display)
     endspent();
 
     /* Request the user's password */
-    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password:"));
+    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
 
     /* If verbose authentication is enabled, output messages from the
      * authentication system */
@@ -79,7 +79,7 @@ gdm_verify_user (const gchar *display)
 
 	if (!pwent) {
 	    gdm_error (_("Couldn't authenticate %s"), login);
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("User unknown"));
+	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
 	    return NULL;
 	}
     
@@ -94,7 +94,7 @@ gdm_verify_user (const gchar *display)
     if (!passwd || !ppasswd || strcmp (crypt (passwd, ppasswd), ppasswd)) {
 
 	if (GdmVerboseAuth)
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Incorrect password"));
+	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
 	
 	return NULL;
     }
