@@ -86,7 +86,7 @@
 #include "misc.h"
 #include "auth.h"
 
-static const gchar RCSid[]="$Id: xdmcp.c,v 1.11 2001/01/23 08:31:29 jirka Exp $";
+static const gchar RCSid[]="$Id: xdmcp.c,v 1.13 2001/02/01 09:47:58 jirka Exp $";
 
 /* TCP Wrapper syslog control */
 gint allow_severity = LOG_INFO;
@@ -111,7 +111,7 @@ extern gint GdmMaxPending;	/* only accept this number of pending sessions */
 extern gint GdmMaxManageWait;	/* Dispose sessions not responding with MANAGE after 10 secs */
 extern gint GdmMaxSessions;	/* Maximum number of remote sessions */
 extern gint GdmPort;		/* UDP port number */
-extern gint GdmIndirect;	/* Honor XDMCP_INDIRECT, i.e. choosing */
+extern gboolean GdmIndirect;	/* Honor XDMCP_INDIRECT, i.e. choosing */
 extern gint GdmMaxIndirectWait;	/* Max wait between INDIRECT_QUERY and MANAGE */
 extern gint GdmDispPerHost;	/* Max number of displays per remote host */
 
@@ -936,7 +936,7 @@ gdm_xdmcp_display_alloc (struct sockaddr_in *clnt_sa, gint displaynum)
     GdmDisplay *d = NULL;
     struct hostent *client_he = NULL;
     
-    d = g_malloc (sizeof (GdmDisplay));
+    d = g_new0 (GdmDisplay, 1);
     d->authfile = NULL;
     d->auths = NULL;
     d->userauth = NULL;
@@ -952,6 +952,8 @@ gdm_xdmcp_display_alloc (struct sockaddr_in *clnt_sa, gint displaynum)
     d->sessionid = globsessid++;
     d->acctime = time (NULL);
     d->dispnum = displaynum;
+
+    d->sleep_before_run = 0;
     
     /* Find client hostname */
     client_he = gethostbyaddr ((gchar *) &clnt_sa->sin_addr,
